@@ -55,7 +55,7 @@ pub fn generate_proxy_getter_impl(m: &Method) -> proc_macro2::TokenStream {
 
     quote! {
         #msig {
-            #module_path Proxy::new_proxy_obj(self.raw_vm_api()) #address_snippet
+            #module_path Proxy::new_proxy_obj() #address_snippet
         }
     }
 }
