@@ -1,12 +1,14 @@
 use super::{
-    arg_regular::*,
     method_call_gen::{
         generate_body_with_result, generate_call_method_body, generate_call_to_method_expr,
     },
     payable_gen::*,
     util::*,
 };
-use crate::model::{ContractTrait, Method, PublicRole, Supertrait};
+use crate::{
+    generate::method_call_gen::generate_arg_nested_tuples,
+    model::{ContractTrait, Method, PublicRole, Supertrait},
+};
 
 /// Callback name max length is checked during derive,
 /// so as not to burden the contract at runtime.
@@ -69,7 +71,6 @@ fn callback_selector_body(
     module_calls: Vec<proc_macro2::TokenStream>,
 ) -> proc_macro2::TokenStream {
     quote! {
-        let mut ___call_result_loader___ = elrond_wasm::io::EndpointDynArgLoader::<Self::Api>::new();
         let ___cb_closure_matcher___ = ___cb_closure___.matcher::<#CALLBACK_NAME_MAX_LENGTH>();
         if ___cb_closure_matcher___.matches_empty() {
             return elrond_wasm::types::CallbackSelectorResult::Processed;
@@ -80,32 +81,46 @@ fn callback_selector_body(
     }
 }
 
+fn load_call_result_args_snippet(m: &Method) -> proc_macro2::TokenStream {
+    // if no `#[call_result]` present, ignore altogether
+    let has_call_result = m
+        .method_args
+        .iter()
+        .any(|arg| arg.metadata.callback_call_result);
+    if !has_call_result {
+        return quote! {};
+    }
+
+    let (call_result_var_names, call_result_var_types, call_result_var_names_str) =
+        generate_arg_nested_tuples(m.method_args.as_slice(), |arg| {
+            arg.metadata.callback_call_result
+        });
+    quote! {
+        let #call_result_var_names = elrond_wasm::io::load_endpoint_args::<Self::Api,#call_result_var_types>(
+            #call_result_var_names_str,
+        );
+    }
+}
+
+fn load_cb_closure_args_snippet(m: &Method) -> proc_macro2::TokenStream {
+    let (closure_var_names, closure_var_types, closure_var_names_str) =
+        generate_arg_nested_tuples(m.method_args.as_slice(), |arg| {
+            arg.is_endpoint_arg() && !arg.metadata.callback_call_result
+        });
+    quote! {
+        let #closure_var_names = elrond_wasm::io::load_multi_args_custom_loader::<Self::Api, _, #closure_var_types>(
+            ___cb_closure___.into_arg_loader(),
+            #closure_var_names_str,
+        );
+    }
+}
+
 fn match_arms(methods: &[Method]) -> Vec<proc_macro2::TokenStream> {
     methods
         .iter()
         .filter_map(|m| {
             if let PublicRole::Callback(callback) = &m.public_role {
                 let payable_snippet = generate_payable_snippet(m);
-                let mut has_call_result = false;
-                let arg_init_snippets: Vec<proc_macro2::TokenStream> = m
-                    .method_args
-                    .iter()
-                    .map(|arg| {
-                        if arg.metadata.payment.is_payment_arg() {
-                            quote! {}
-                        } else if arg.metadata.callback_call_result {
-                            has_call_result = true;
-
-                            // Should be an AsyncCallResult argument that wraps what comes from the async call.
-                            // But in principle, one can express it it any way.
-                            generate_load_dyn_arg(arg, &quote! { &mut ___call_result_loader___ })
-                        } else {
-                            // callback args, loaded from storage via the tx hash
-                            generate_load_dyn_arg(arg, &quote! { &mut ___cb_arg_loader___ })
-                        }
-                    })
-                    .collect();
-
                 let callback_name_str = &callback.callback_name.to_string();
                 assert!(
                     callback_name_str.len() <= CALLBACK_NAME_MAX_LENGTH,
@@ -114,23 +129,16 @@ fn match_arms(methods: &[Method]) -> Vec<proc_macro2::TokenStream> {
                     CALLBACK_NAME_MAX_LENGTH
                 );
                 let callback_name_literal = byte_str_literal(callback_name_str.as_bytes());
+                let load_call_result_args = load_call_result_args_snippet(m);
+                let load_cb_closure_args = load_cb_closure_args_snippet(m);
                 let call = generate_call_to_method_expr(m);
-                let call_result_assert_no_more_args = if has_call_result {
-                    quote! {
-                        elrond_wasm::io::assert_no_more_args::<Self::Api, _>(&___call_result_loader___);
-                    }
-                } else {
-                    quote! {}
-                };
                 let body_with_result = generate_body_with_result(&m.return_type, &call);
 
                 let match_arm = quote! {
                     else if ___cb_closure_matcher___.name_matches(#callback_name_literal) {
                         #payable_snippet
-                        let mut ___cb_arg_loader___ = ___cb_closure___.into_arg_loader();
-                        #(#arg_init_snippets)*
-                        elrond_wasm::io::assert_no_more_args::<Self::Api, _>(&___cb_arg_loader___);
-                        #call_result_assert_no_more_args
+                        #load_call_result_args
+                        #load_cb_closure_args
                         #body_with_result ;
                         return elrond_wasm::types::CallbackSelectorResult::Processed;
                     }
