@@ -1,5 +1,4 @@
 pub mod abi_gen;
-pub mod arg_regular;
 pub mod auto_impl;
 pub mod auto_impl_event;
 pub mod auto_impl_proxy;
@@ -7,6 +6,7 @@ pub mod auto_impl_storage;
 pub mod callback_gen;
 pub mod callback_proxies_gen;
 pub mod contract_gen;
+pub mod convert_to_owned_type;
 pub mod endpoints_mod_gen;
 pub mod function_selector;
 pub mod method_call_gen;
