@@ -1,45 +1,73 @@
-use super::{method_gen::*, util::*};
+use quote::ToTokens;
+
+use super::util::*;
 use crate::{
-    generate::{snippets, supertrait_gen},
-    model::{ArgPaymentMetadata, ContractTrait, Method, PublicRole},
+    generate::{convert_to_owned_type::convert_to_owned_type, snippets, supertrait_gen},
+    model::{ArgPaymentMetadata, ContractTrait, Method, MethodArgument, PublicRole},
 };
 
-pub fn generate_proxy_endpoint_sig(method: &Method) -> proc_macro2::TokenStream {
+pub fn proxy_arg_gen(
+    method_args: &[MethodArgument],
+    generics: &mut syn::Generics,
+) -> Vec<proc_macro2::TokenStream> {
+    let mut args_decl = Vec::new();
+    for (arg_index, arg) in method_args.iter().enumerate() {
+        let unprocessed_attributes = &arg.unprocessed_attributes;
+        let pat = &arg.pat;
+        if arg.is_endpoint_arg() {
+            // let ty = &arg.ty;
+            let mut bounds = syn::punctuated::Punctuated::new();
+            bounds.push(syn::TypeParamBound::Trait(syn::TraitBound {
+                paren_token: None,
+                modifier: syn::TraitBoundModifier::None,
+                lifetimes: None,
+                path: equivalent_encode_path_gen(&arg.ty),
+            }));
+            let arg_type_generated_ident = generate_proxy_type_generic(arg_index);
+            generics
+                .params
+                .push(syn::GenericParam::Type(syn::TypeParam {
+                    attrs: Vec::new(),
+                    ident: arg_type_generated_ident.clone(),
+                    colon_token: Some(syn::token::Colon(proc_macro2::Span::call_site())),
+                    bounds,
+                    eq_token: None,
+                    default: None,
+                }));
+            args_decl.push(quote! { #(#unprocessed_attributes)* #pat : #arg_type_generated_ident });
+        } else {
+            let ty = &arg.ty;
+            args_decl.push(quote! { #(#unprocessed_attributes)* #pat : #ty });
+        }
+    }
+
+    args_decl
+}
+
+pub fn generate_proxy_method_sig(
+    method: &Method,
+    proxy_return_struct_path: proc_macro2::TokenStream,
+) -> proc_macro2::TokenStream {
     let method_name = &method.name;
-    let generics = &method.generics;
+    let mut generics = method.generics.clone();
     let generics_where = &method.generics.where_clause;
-    let arg_decl = arg_declarations(&method.method_args);
+    let arg_decl = proxy_arg_gen(&method.method_args, &mut generics);
     let ret_tok = match &method.return_type {
         syn::ReturnType::Default => quote! { () },
         syn::ReturnType::Type(_, ty) => quote! { #ty },
     };
     let result = quote! {
         fn #method_name #generics (
-            self,
+            &mut self,
             #(#arg_decl),*
-        ) -> elrond_wasm::types::ContractCall<Self::Api, <#ret_tok as elrond_wasm::elrond_codec::TopEncodeMulti>::DecodeAs>
-        #generics_where
-    };
-    result
-}
-
-pub fn generate_proxy_deploy_sig(method: &Method) -> proc_macro2::TokenStream {
-    let method_name = &method.name;
-    let generics = &method.generics;
-    let generics_where = &method.generics.where_clause;
-    let arg_decl = arg_declarations(&method.method_args);
-    let result = quote! {
-        fn #method_name #generics (
-            self,
-            #(#arg_decl),*
-        ) -> elrond_wasm::types::ContractDeploy<Self::Api>
+        ) -> #proxy_return_struct_path<Self::Api, #ret_tok>
         #generics_where
     };
     result
 }
 
 pub fn generate_proxy_endpoint(m: &Method, endpoint_name: String) -> proc_macro2::TokenStream {
-    let msig = generate_proxy_endpoint_sig(m);
+    let msig = generate_proxy_method_sig(m, quote! { elrond_wasm::types::ContractCall });
 
     let mut token_count = 0;
     let mut token_expr = quote! { elrond_wasm::types::TokenIdentifier::<Self::Api>::egld() };
@@ -130,7 +158,7 @@ pub fn generate_proxy_endpoint(m: &Method, endpoint_name: String) -> proc_macro2
         #[allow(clippy::too_many_arguments)]
         #[allow(clippy::type_complexity)]
         #msig {
-            let ___address___ = self.into_fields();
+            let ___address___ = self.extract_address();
             let mut ___contract_call___ = elrond_wasm::types::new_contract_call(
                 ___address___,
                 #endpoint_name_literal,
@@ -147,7 +175,8 @@ pub fn generate_proxy_endpoint(m: &Method, endpoint_name: String) -> proc_macro2
 }
 
 pub fn generate_proxy_deploy(init_method: &Method) -> proc_macro2::TokenStream {
-    let msig = generate_proxy_deploy_sig(init_method);
+    let msig =
+        generate_proxy_method_sig(init_method, quote! { elrond_wasm::types::ContractDeploy });
 
     let mut payment_count = 0;
     let mut multi_count = 0;
@@ -202,9 +231,9 @@ pub fn generate_proxy_deploy(init_method: &Method) -> proc_macro2::TokenStream {
         #[allow(clippy::too_many_arguments)]
         #[allow(clippy::type_complexity)]
         #msig {
-            let ___address___ = self.into_fields();
-            let mut ___contract_deploy___ = elrond_wasm::types::new_contract_deploy::<Self::Api>(
-                ___address___,
+            let ___opt_address___ = self.extract_opt_address();
+            let mut ___contract_deploy___ = elrond_wasm::types::new_contract_deploy(
+                ___opt_address___,
             );
             #(#arg_push_snippets)*
             ___contract_deploy___
@@ -254,3 +283,15 @@ pub fn proxy_obj_code(contract: &ContractTrait) -> proc_macro2::TokenStream {
         #(#impl_all_proxy_traits)*
     }
 }
+
+fn equivalent_encode_path_gen(ty: &syn::Type) -> syn::Path {
+    let owned_type = convert_to_owned_type(ty);
+    syn::parse_str(
+        format!(
+            "elrond_wasm::elrond_codec::CodecInto<{}>",
+            owned_type.to_token_stream()
+        )
+        .as_str(),
+    )
+    .unwrap()
+}
