@@ -21,12 +21,14 @@ impl LegacyEventAttribute {
         match is_attr_one_string_arg(attr, ATTR_LEGACY_EVENT) {
             None => None,
             Some(event_str) => {
-                if !event_str.starts_with("0x") {
-                    panic!("event id should start with '0x'");
-                }
-                if event_str.len() != 64 + 2 {
-                    panic!("event id should be 64 characters long (32 bytes)");
-                }
+                assert!(
+                    event_str.starts_with("0x"),
+                    "event id should start with '0x'"
+                );
+                assert!(
+                    event_str.len() == 64 + 2,
+                    "event id should be 64 characters long (32 bytes)"
+                );
                 let substr = &event_str[2..];
                 let result_str = substr.to_string();
                 match hex::decode(result_str) {
